//
//  MultipleSurfaceView.m
//  SciChartDemo
//
//  Created by Admin on 14.04.16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

#import "MultipleSurfaceChartView.h"
#import <SciChart/SciChart.h>
#import "DataManager.h"

@interface RangeSync : NSObject
@end


@interface MultipleSurfaceChartView () {
    SCIMultiSurfaceModifier * szpm;
    SCIMultiSurfaceModifier *szem;
    SCIMultiSurfaceModifier *spzm;
    
    SCIMultiSurfaceModifier * sXPinch;
    SCIMultiSurfaceModifier *sYPinch;
    SCIMultiSurfaceModifier *sXDrag;
    SCIMultiSurfaceModifier *sYDrag;
    
    SCIMultiSurfaceModifier *sRollover;
    
    SCIAxisRangeSyncronization * rSync;
    SCIAxisAreaSizeSyncronization * sSync;
}

@property (nonatomic, weak) SCIChartSurfaceView * sciChartView1;
@property (nonatomic, weak) SCIChartSurfaceView * sciChartView2;

@property (nonatomic, strong) SCIChartSurface * surface1;
@property (nonatomic, strong) SCIChartSurface * surface2;

@end

@implementation MultipleSurfaceChartView

-(instancetype)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    
    if (self) {
        SCIChartSurfaceView * view = [[SCIChartSurfaceView alloc]init];
        _sciChartView1 = view;
        [_sciChartView1 setTranslatesAutoresizingMaskIntoConstraints:NO];
        [self addSubview:_sciChartView1];
        
        view = [[SCIChartSurfaceView alloc]init];
        _sciChartView2 = (SCIChartSurfaceView*)view;
        [_sciChartView2 setTranslatesAutoresizingMaskIntoConstraints:NO];
        [self addSubview:_sciChartView2];
        NSDictionary *layout = @{@"SciChart1":_sciChartView1, @"SciChart2":_sciChartView2};
        
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"|-(0)-[SciChart1]-(0)-|" options:0 metrics:0 views:layout]];
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"|-(0)-[SciChart2]-(0)-|" options:0 metrics:0 views:layout]];
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:|-(0)-[SciChart1(SciChart2)]-(10)-[SciChart2(SciChart1)]-(0)-|" options:0 metrics:0 views:layout]];
        
        [self createMultiSurface];
        [self createSharedModifiers];
        [self initializeSurface1Data];
        [self initializeSurface2Data];
    }
    
    return self;
}

-(void) createSharedModifiers {
    rSync = [SCIAxisRangeSyncronization new];
    sSync = [SCIAxisAreaSizeSyncronization new];
    sSync.syncMode = SCIAxisSizeSync_Right;
    
    szpm = [[SCIMultiSurfaceModifier alloc] initWithModifierType:[SCIZoomPanModifier class]];
    szem = [[SCIMultiSurfaceModifier alloc] initWithModifierType:[SCIZoomExtentsModifier class]];
    spzm = [[SCIMultiSurfaceModifier alloc] initWithModifierType:[SCIPinchZoomModifier class]];
    
    sXPinch = [[SCIMultiSurfaceModifier alloc] initWithModifierType:[SCIAxisPinchZoomModifier class]];
    sYPinch = [[SCIMultiSurfaceModifier alloc] initWithModifierType:[SCIAxisPinchZoomModifier class]];
    sXDrag = [[SCIMultiSurfaceModifier alloc] initWithModifierType:[SCIXAxisDragModifier class]];
    sYDrag = [[SCIMultiSurfaceModifier alloc] initWithModifierType:[SCIYAxisDragModifier class]];
    
    sRollover = [[SCIMultiSurfaceModifier alloc] initWithModifierType:[SCIRolloverModifier class]];
}

-(void) createMultiSurface {
    _surface1 = [[SCIChartSurface alloc] initWithView: _sciChartView1];
    [[_surface1 style] setBackgroundBrush: [[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
    [[_surface1 style] setSeriesBackgroundBrush:[[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];

    _surface2 = [[SCIChartSurface alloc] initWithView: _sciChartView2];
    [[_surface2 style] setBackgroundBrush: [[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
    [[_surface2 style] setSeriesBackgroundBrush:[[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
    
    [sSync attachSurface:_surface1];
    [sSync attachSurface:_surface2];
}

-(void) initializeSurface1Data {
    SCIPenSolid  *majorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF393532 Width:0.5];
    SCIBrushSolid  *gridBandPen = [[SCIBrushSolid alloc] initWithColorCode:0xE1232120];
    SCIPenSolid  *minorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF262423 Width:0.5];
    SCITextFormattingStyle *  textFormatting = [[SCITextFormattingStyle alloc] init];
    [textFormatting setFontSize:16];
    [textFormatting setFontName:@"Helvetica"];
    [textFormatting setColorCode:0xFFafb3b6];
    
    SCIAxisStyle * axisStyle = [[SCIAxisStyle alloc]init];
    [axisStyle setMajorTickBrush:majorPen];
    [axisStyle setGridBandBrush: gridBandPen];
    [axisStyle setMajorGridLineBrush:majorPen];
    [axisStyle setMinorTickBrush:minorPen];
    [axisStyle setMinorGridLineBrush:minorPen];
    [axisStyle setLabelStyle:textFormatting ];
    [axisStyle setDrawMinorGridLines:YES];
    [axisStyle setDrawMajorBands:YES];
    
    id<SCIAxis2D> axis = [[SCINumericAxis alloc] init];
    [axis setStyle: axisStyle];
    axis.axisId = @"Y1";
    [axis setGrowBy: [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0.1) Max:SCIGeneric(0.1)]];
    [_surface1 attachAxis:axis IsXAxis:NO];
    
    axis = [[SCINumericAxis alloc] init];
    axis.axisId = @"X1";
    [axis setStyle: axisStyle];
    [axis setGrowBy: [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0.1) Max:SCIGeneric(0.1)]];
    [_surface1 attachAxis:axis IsXAxis:YES];
    [rSync attachAxis:axis];
    
//    SCIAxisPinchZoomModifier * x1Pinch = [SCIAxisPinchZoomModifier new];
    SCIAxisPinchZoomModifier * x1Pinch = [sXPinch modifierForSurface:_surface1];
    x1Pinch.axisId = @"X1";
//    SCIXAxisDragModifier * x1Drag = [SCIXAxisDragModifier new];
    SCIXAxisDragModifier * x1Drag = [sXDrag modifierForSurface:_surface1];
    x1Drag.axisId = @"X1";
    x1Drag.dragMode = SCIAxisDragMode_Scale;
    x1Drag.clipModeX = SCIZoomPanClipMode_None;
    
    SCIAxisPinchZoomModifier * y1Pinch = [sYPinch modifierForSurface:_surface1];//[SCIAxisPinchZoomModifier new];
    y1Pinch.axisId = @"Y1";
    SCIYAxisDragModifier * y1Drag = [sYDrag modifierForSurface:_surface1];//[SCIYAxisDragModifier new];
    y1Drag.axisId = @"Y1";
    y1Drag.dragMode = SCIAxisDragMode_Pan;
    
    SCIRolloverModifier * rollover = [sRollover modifierForSurface:_surface1];
    [rollover.style setRolloverPen:[[SCIPenSolid alloc] initWithColorCode:0xA099EE99 Width:1.5]];
    
    
    SCIPinchZoomModifier * pzm = [[SCIPinchZoomModifier alloc] init];
    
    SCIModifierGroup * gm = [[SCIModifierGroup alloc] initWithChildModifiers:@[sXPinch, sYPinch, sXDrag, sYDrag,
                                                                               pzm, szem, sRollover]];
    _surface1.chartModifier = gm;
    
    SCIXyDataSeries * priceDataSeries = [[SCIXyDataSeries alloc] initWithXType:SCIDataType_Float YType:SCIDataType_Float];
    int dataCount = 20;
    //Getting Fourier dataSeries
    for (int j = 0; j < dataCount; j++) {
        double time = 10 * j / (double)dataCount;
        double x = time;
        double y = arc4random_uniform(20);
        [priceDataSeries appendX:SCIGeneric(x) Y:SCIGeneric(y)];
    }
    priceDataSeries.dataDistributionCalculator = [SCIUserDefinedDistributionCalculator new];
    
    SCIEllipsePointMarker * ellipsePointMarker = [[SCIEllipsePointMarker alloc]init];
    [ellipsePointMarker setDrawBorder:YES];
    [ellipsePointMarker setFillBrush:[[SCIBrushSolid alloc] initWithColorCode:0xFFd6ffd7]];
    [ellipsePointMarker setHeight:5];
    [ellipsePointMarker setWidth:5];
    
    SCIFastLineRenderableSeries * priceRenderableSeries = [SCIFastLineRenderableSeries new];
    [priceRenderableSeries.style setPointMarker: ellipsePointMarker];
    [priceRenderableSeries.style setDrawPointMarkers: YES];
    [priceRenderableSeries.style setLinePen: [[SCIPenSolid alloc] initWithColorCode:0xFF99EE99 Width:0.7]];
    
    [priceRenderableSeries setXAxisId: @"X1"];
    [priceRenderableSeries setYAxisId: @"Y1" ];
    [priceRenderableSeries setDataSeries:priceDataSeries];
    
    [_surface1 attachRenderableSeries:priceRenderableSeries];
    
    [_surface1 invalidateElement];
}

-(void) initializeSurface2Data {
    SCIPenSolid  *majorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF393532 Width:0.5];
    SCIBrushSolid  *gridBandPen = [[SCIBrushSolid alloc] initWithColorCode:0xE1232120];
    SCIPenSolid  *minorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF262423 Width:0.5];
    
    SCITextFormattingStyle *  textFormatting= [[SCITextFormattingStyle alloc] init];
    [textFormatting setFontSize:16];
    [textFormatting setFontName:@"Helvetica"];
    [textFormatting setColorCode:0xFFafb3b6];
    
    SCIAxisStyle * axisStyle = [[SCIAxisStyle alloc]init];
    [axisStyle setMajorTickBrush:majorPen];
    [axisStyle setGridBandBrush: gridBandPen];
    [axisStyle setMajorGridLineBrush:majorPen];
    [axisStyle setMinorTickBrush:minorPen];
    [axisStyle setMinorGridLineBrush:minorPen];
    [axisStyle setLabelStyle:textFormatting];
    [axisStyle setDrawMinorGridLines:YES];
    [axisStyle setDrawMajorBands:YES];
    
    id<SCIAxis2D> axis = [[SCINumericAxis alloc] init];
    axis.axisId = @"Y2";
    [axis setStyle: axisStyle];
    [axis setGrowBy: [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0.1) Max:SCIGeneric(0.1)]];
    [_surface2 attachAxis:axis IsXAxis:NO];
    
    axis = [[SCINumericAxis alloc] init];
    axis.axisId = @"X2";
    [rSync attachAxis:axis];
    [axis setStyle: axisStyle];
    [axis setGrowBy: [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0.1) Max:SCIGeneric(0.1)]];
    [_surface2 attachAxis:axis IsXAxis:YES];
    
    SCIAxisPinchZoomModifier * x2Pinch = [sXPinch modifierForSurface:_surface2];//[SCIAxisPinchZoomModifier new];
    x2Pinch.axisId = @"X2";
    SCIXAxisDragModifier * x2Drag = [sXDrag modifierForSurface:_surface2];//[SCIXAxisDragModifier new];
    x2Drag.axisId = @"X2";
    x2Drag.dragMode = SCIAxisDragMode_Scale;
    x2Drag.clipModeX = SCIZoomPanClipMode_None;
    
    SCIAxisPinchZoomModifier * y2Pinch = [sYPinch modifierForSurface:_surface2];//[SCIAxisPinchZoomModifier new];
    y2Pinch.axisId = @"Y2";
    SCIYAxisDragModifier * y2Drag = [sYDrag modifierForSurface:_surface2];//[SCIYAxisDragModifier new];
    y2Drag.axisId = @"Y2";
    y2Drag.dragMode = SCIAxisDragMode_Pan;
    
    SCIRolloverModifier * rollover = [sRollover modifierForSurface:_surface2];
    [rollover.style setRolloverPen:[[SCIPenSolid alloc] initWithColorCode:0xA04c8aff Width:1.5]];
    
    SCIPinchZoomModifier * pzm = [[SCIPinchZoomModifier alloc] init];
    
    SCIModifierGroup * gm = [[SCIModifierGroup alloc] initWithChildModifiers:@[sXPinch, sYPinch, sXDrag, sYDrag,
                                                                               pzm, szem, sRollover]];
    _surface2.chartModifier = gm;
    
    SCIXyDataSeries * fourierDataSeries = [[SCIXyDataSeries alloc] initWithXType:SCIDataType_Float YType:SCIDataType_Float];
    int dataCount = 1000;
    //Getting Fourier dataSeries
    for (int j = 0; j < dataCount; j++) {
        double time = 10 * j / (double)dataCount;
        double x = time;
        double y = 2 * sin(x)+10;
        [fourierDataSeries appendX:SCIGeneric(x) Y:SCIGeneric(y)];
    }
    fourierDataSeries.dataDistributionCalculator = [SCIUserDefinedDistributionCalculator new];
    
    SCIFastLineRenderableSeries * fourierRenderableSeries = [SCIFastLineRenderableSeries new];
    fourierRenderableSeries.style.linePen = [[SCIPenSolid alloc] initWithColorCode:0xFF4c8aff Width:0.7];
    
    fourierRenderableSeries.xAxisId = @"X2";
    fourierRenderableSeries.yAxisId = @"Y2";
    [fourierRenderableSeries setDataSeries:fourierDataSeries];
    
    [_surface2 attachRenderableSeries:fourierRenderableSeries];

    [_surface2 invalidateElement];
}

@end
