//
//  ScatterSeriesViewController.m
//  SciChartDemo
//
//  Created by Admin on 27.01.16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

#import "ScatterSeriesChartView.h"
#import <SciChart/SciChart.h>
#import "DataManager.h"

@implementation ScatterSeriesChartView

@synthesize sciChartSurfaceView;
@synthesize surface;

-(id<SCIRenderableSeries>) getScatterRenderableSeriesWithDetalization:(int)pointMarkerDetalization
                                                                Color:(unsigned int) color
                                                             Negative:(BOOL) negative
{
    SCIXyDataSeries * scatterDataSeries = [[SCIXyDataSeries alloc] initWithXType:SCIDataType_DateTime YType:SCIDataType_Float];
    
    //Getting Fourier dataSeries
    for (int i = 0; i < 200; i++) {
        double x = i;
        double time =  (i < 100) ? arc4random_uniform(x+10) : arc4random_uniform(200-x+10);
        double y = time*time*time;
        if (negative) {
            [scatterDataSeries appendX:SCIGeneric(x) Y:SCIGeneric(-y)];
        } else {
            [scatterDataSeries appendX:SCIGeneric(x) Y:SCIGeneric(y)];
        }
    }
    
    SCIXyScatterRenderableSeries * xyScatterRenderableSeries = [[SCIXyScatterRenderableSeries alloc] init];
    scatterDataSeries.dataDistributionCalculator = [SCIUserDefinedDistributionCalculator new];
    scatterDataSeries.seriesName = (pointMarkerDetalization == 6)
    ? ( (negative)
       ? @"Negative Hex"
       : @"Positive Hex" )
    : ( (negative)
       ? @"Negative"
       : @"Positive" );
    
    SCIEllipsePointMarker * ellipsePointMarker = [[SCIEllipsePointMarker alloc]init];
    ellipsePointMarker.drawBorder = YES;
    ellipsePointMarker.fillBrush = [[SCIBrushSolid alloc] initWithColorCode:color];
    ellipsePointMarker.borderPen = [[SCIPenSolid alloc] initWithColorCode:0xFFFFFFFF Width:0.1];
    ellipsePointMarker.detalization = pointMarkerDetalization;
    ellipsePointMarker.height = 6;
    ellipsePointMarker.width = 6;
    
    xyScatterRenderableSeries.style.pointMarker = ellipsePointMarker;
    xyScatterRenderableSeries.xAxisId = @"xAxis";
    xyScatterRenderableSeries.yAxisId = @"yAxis";
    xyScatterRenderableSeries.dataSeries = scatterDataSeries;
    
    return xyScatterRenderableSeries;
}

-(instancetype)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    
    if (self) {
        SCIChartSurfaceView * view = [[SCIChartSurfaceView alloc]init];
        sciChartSurfaceView = view;
        
        [sciChartSurfaceView setTranslatesAutoresizingMaskIntoConstraints:NO];
        
        [self addSubview:sciChartSurfaceView];
        NSDictionary *layout = @{@"SciChart":sciChartSurfaceView};
        
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"|-(0)-[SciChart]-(0)-|" options:0 metrics:0 views:layout]];
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:|-(0)-[SciChart]-(0)-|" options:0 metrics:0 views:layout]];
        
        [self initializeSurfaceData];
    }
    
    return self;
}

-(void) initializeSurfaceData {
    [surface free];
    surface = [[SCIChartSurface alloc] initWithView: sciChartSurfaceView];
    
    [[surface style] setBackgroundBrush: [[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
    [[surface style] setSeriesBackgroundBrush:[[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
    
    SCIPenSolid *majorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF393532 Width:0.6];
    SCIBrushSolid  *gridBandPen = [[SCIBrushSolid alloc] initWithColorCode:0xE1232120];
    SCIPenSolid  *minorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF262423 Width:0.5];
    
    SCITextFormattingStyle *  textFormatting= [[SCITextFormattingStyle alloc] init];
    [textFormatting setFontSize:16];
    [textFormatting setFontName:@"Arial"];
    [textFormatting setColorCode:0xFFafb3b6];
    
    SCIAxisStyle * axisStyle = [[SCIAxisStyle alloc]init];
    [axisStyle setMajorTickBrush:majorPen];
    [axisStyle setGridBandBrush: gridBandPen];
    [axisStyle setMajorGridLineBrush:majorPen];
    [axisStyle setMinorTickBrush:minorPen];
    [axisStyle setMinorGridLineBrush:minorPen];
    [axisStyle setLabelStyle:textFormatting ];
    [axisStyle setDrawMinorGridLines:YES];
    [axisStyle setDrawMajorBands:YES];
    
    id<SCIAxis2D> axis = [[SCINumericAxis alloc] init];
    [axis setStyle: axisStyle];
    axis.axisId = @"yAxis";
    [surface attachAxis:axis IsXAxis:NO];
    [axis setGrowBy: [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0.1) Max:SCIGeneric(0.1)]];
    
    axis = [[SCIDateTimeAxis alloc] init];
    axis.axisId = @"xAxis";
    [axis setStyle: axisStyle];
    [surface attachAxis:axis IsXAxis:YES];
    [axis setGrowBy: [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0.1) Max:SCIGeneric(0.1)]];
    
    SCIXAxisDragModifier * xDragModifier = [SCIXAxisDragModifier new];
    xDragModifier.axisId = @"xAxis";
    xDragModifier.dragMode = SCIAxisDragMode_Scale;
    xDragModifier.clipModeX = SCIZoomPanClipMode_None;
    
    SCIYAxisDragModifier * yDragModifier = [SCIYAxisDragModifier new];
    yDragModifier.axisId = @"yAxis";
    yDragModifier.dragMode = SCIAxisDragMode_Pan;
    
    
    SCIPinchZoomModifier * pzm = [[SCIPinchZoomModifier alloc] init];
    SCIZoomExtentsModifier * zem = [[SCIZoomExtentsModifier alloc] init];
    
    SCICursorModifier * cursor = [[SCICursorModifier alloc] init];
    cursor.style.hitTestMode = SCIHitTest_Point;
    cursor.style.colorMode = SCITooltipColorMode_SeriesColorToDataView;
    [cursor.style setTooltipSize:CGSizeMake(200, NAN)];
    
    [zem setModifierName:@"ZoomExtents Modifier"];
    [pzm setModifierName:@"PinchZoom Modifier"];
    [cursor setModifierName:@"Cursor Modifier"];
    [yDragModifier setModifierName:@"Y Axis Drag Modifier"];
    [xDragModifier setModifierName:@"X Axis Drag Modifier"];

    
    SCIModifierGroup * gm = [[SCIModifierGroup alloc] initWithChildModifiers:@[xDragModifier, yDragModifier, pzm, zem, cursor]];
    surface.chartModifier = gm;
    
    id<SCIRenderableSeries> chart1 = [self getScatterRenderableSeriesWithDetalization:3 Color:0xFF01ebff Negative:NO];
    id<SCIRenderableSeries> chart2 = [self getScatterRenderableSeriesWithDetalization:6 Color:0xFF00a3ff Negative:NO];
    id<SCIRenderableSeries> chart3 = [self getScatterRenderableSeriesWithDetalization:3 Color:0xFF0165ff Negative:YES];
    id<SCIRenderableSeries> chart4 = [self getScatterRenderableSeriesWithDetalization:6 Color:0xFF00a3ff Negative:YES];
    
    [surface attachRenderableSeries:chart1];
    [surface attachRenderableSeries:chart2];
    [surface attachRenderableSeries:chart3];
    [surface attachRenderableSeries:chart4];
    
    [surface invalidateElement];
}

@end
