//
//  ToolTipCustomization.m
//  SciChartDemo
//
//  Created by Mykola Hrybeniuk on 8/26/16.
//  Copyright © 2016 ABT. All rights reserved.
//

#import "UsingTooltipModifierChartView.h"
#import <SciChart/SciChart.h>
#import "DataManager.h"

@implementation UsingTooltipModifierChartView

@synthesize sciChartSurfaceView;
@synthesize surface;

- (void)addModifiers{
    SCITooltipModifier *toolTipModifier = [[SCITooltipModifier alloc] init];
    toolTipModifier.style.colorMode = SCITooltipColorMode_SeriesColorToDataView;
    self.surface.chartModifier = toolTipModifier;
}

- (instancetype)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    
    if (self) {
        SCIChartSurfaceView * view = [[SCIChartSurfaceView alloc]initWithFrame:frame];
        sciChartSurfaceView = view;
        
        [sciChartSurfaceView setTranslatesAutoresizingMaskIntoConstraints:NO];
        
        [self addSubview:sciChartSurfaceView];
        NSDictionary *layout = @{@"SciChart":sciChartSurfaceView};
        
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"|-(0)-[SciChart]-(0)-|" options:0 metrics:0 views:layout]];
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:|-(0)-[SciChart]-(0)-|" options:0 metrics:0 views:layout]];
        
        [self initializeSurfaceData];
    }
    
    return self;
}

- (void)initializeSurfaceData {
    surface = [[SCIChartSurface alloc] initWithView: sciChartSurfaceView];
    
    [[surface style] setBackgroundBrush: [[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
    [[surface style] setSeriesBackgroundBrush:[[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
    [self addAxes];
    [self addModifiers];
    [self initializeSurfaceRenderableSeries];
}

- (void)addAxes{
    SCIPenSolid * majorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF393532 Width:0.5];
    SCIBrushSolid * gridBandPen = [[SCIBrushSolid alloc] initWithColorCode:0xE1232120];
    SCIPenSolid * minorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF262423 Width:0.5];
    
    SCITextFormattingStyle *  textFormatting= [[SCITextFormattingStyle alloc] init];
    [textFormatting setFontSize:16];
    [textFormatting setFontName:@"Helvetica"];
    [textFormatting setColorCode:0xFFafb3b6];
    
    SCIAxisStyle * axisStyle = [[SCIAxisStyle alloc]init];
    [axisStyle setMajorTickBrush:majorPen];
    [axisStyle setGridBandBrush: gridBandPen];
    [axisStyle setMajorGridLineBrush:majorPen];
    [axisStyle setMinorTickBrush:minorPen];
    [axisStyle setMinorGridLineBrush:minorPen];
    [axisStyle setLabelStyle:textFormatting ];
    [axisStyle setDrawMinorGridLines:YES];
    [axisStyle setDrawMajorBands:YES];
    
    id<SCIAxis2D> axis = [[SCINumericAxis alloc] init];
    [axis setStyle: axisStyle];
    axis.axisId = @"yAxis";
    [axis setAutoRange:SCIAutoRange_Never];
    [axis setVisibleRange:[[SCIDoubleRange alloc]initWithMin:SCIGeneric(-1.5) Max:SCIGeneric(1.5)]];
    [surface attachAxis:axis IsXAxis:NO];
    
    axis = [[SCINumericAxis alloc] init];
    axis.axisId = @"xAxis";
    [axis setStyle: axisStyle];
    [axis setVisibleRange:[[SCIDoubleRange alloc]initWithMin:SCIGeneric(0) Max:SCIGeneric(10)]];
    [axis setAutoRange:SCIAutoRange_Never];
    [surface attachAxis:axis IsXAxis:YES];
    
}

- (void)initializeSurfaceRenderableSeries{
    [self attachLissajousCurveSeries];
    [self attachSinewaveSeries];
}

- (void)attachSinewaveSeries {
    
    int dataCount = 500;
    int freq = 10;
    double amplitude = 1.5f;
    double phase = 1.0f;
    
    SCIXyDataSeries * dataSeries = [[SCIXyDataSeries alloc] initWithXType:SCIDataType_Double YType:SCIDataType_Double];
    dataSeries.dataDistributionCalculator = [SCIUserDefinedDistributionCalculator new];
    dataSeries.seriesName = @"Sinewave";
    
    for (int i = 0; i < dataCount; i++) {
        double x = 10 * i / (double)dataCount;
        double wn = 2 * M_PI / (dataCount / (double)freq);
        double y = amplitude * sin(i*wn+phase);
        [dataSeries appendX:SCIGeneric(x) Y:SCIGeneric(y)];
    }
    
    SCIFastLineRenderableSeries * rSeries = [[SCIFastLineRenderableSeries alloc] init];
    rSeries.style.linePen = [[SCIPenSolid alloc] initWithColor:[UIColor colorWithRed:255.f/255.f
                                                                               green:51.f/255.f
                                                                                blue:51.f/255.f
                                                                               alpha:1.f]
                                                         Width:0.5];
    [rSeries setXAxisId: @"xAxis"];
    [rSeries setYAxisId: @"yAxis"];
    
    SCIEllipsePointMarker * ellipsePointMarker = [[SCIEllipsePointMarker alloc]init];
    [ellipsePointMarker setDrawBorder:NO];
    [ellipsePointMarker setFillBrush:[[SCIBrushSolid alloc] initWithColor:[UIColor colorWithRed:255.f/255.f
                                                                                          green:51.f/255.f
                                                                                           blue:51.f/255.f
                                                                                          alpha:1.f]]];
    [ellipsePointMarker setHeight:5];
    [ellipsePointMarker setWidth:5];
    rSeries.style.drawPointMarkers = YES;
    rSeries.style.pointMarker = ellipsePointMarker;
    rSeries.dataSeries = dataSeries;
    
    [surface attachRenderableSeries:rSeries];
    [surface invalidateElement];
}

- (void)attachLissajousCurveSeries {
    
    int dataCount = 500;
    float alpha = 0.8f;
    float beta = 0.2f;
    float delta = 0.43f;
    
    SCIXyDataSeries * dataSeries = [[SCIXyDataSeries alloc] initWithXType:SCIDataType_Double YType:SCIDataType_Double];
    dataSeries.acceptUnsortedData = YES;
    SCIUserDefinedDistributionCalculator *distributionCalculator = [SCIUserDefinedDistributionCalculator new];
    dataSeries.dataDistributionCalculator = distributionCalculator;
    dataSeries.seriesName = @"Lissajou";
    
    for (int i = 0; i < dataCount; i++) {
        double x = sin(alpha*i*0.1 + delta);
        double y = sin(beta*i*0.1);
        [dataSeries appendX:SCIGeneric((x+1.f)*5.f) Y:SCIGeneric(y)];
    }
    
    SCIFastLineRenderableSeries * rSeries = [[SCIFastLineRenderableSeries alloc] init];
    rSeries.style.linePen = [[SCIPenSolid alloc] initWithColor:[UIColor colorWithRed:70.f/255.f
                                                                               green:130.f/255.f
                                                                                blue:180.f/255.f
                                                                               alpha:1.f]
                                                         Width:0.5];
    [rSeries setXAxisId: @"xAxis"];
    [rSeries setYAxisId: @"yAxis"];
    
    SCIEllipsePointMarker * ellipsePointMarker = [[SCIEllipsePointMarker alloc]init];
    [ellipsePointMarker setDrawBorder:NO];
    [ellipsePointMarker setFillBrush:[[SCIBrushSolid alloc] initWithColor:[UIColor colorWithRed:70.f/255.f
                                                                                          green:130.f/255.f
                                                                                           blue:180.f/255.f
                                                                                          alpha:1.f]]];
    [ellipsePointMarker setHeight:5];
    [ellipsePointMarker setWidth:5];
    rSeries.style.drawPointMarkers = YES;
    rSeries.style.pointMarker = ellipsePointMarker;
    rSeries.dataSeries = dataSeries;
    
    [surface attachRenderableSeries:rSeries];
    [surface invalidateElement];
    
}

@end
