//
//  SCSECGChartView.swift
//  SciChartSwiftDemo
//
//  Created by Mykola Hrybeniuk on 6/6/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import Foundation
import SciChart

class SCSECGChartView: SCSBaseChartView {
    
    let dataSeries: SCIXyDataSeries = SCIXyDataSeries(fifoWithXType: .Float, YType: .Float, fifoSize: 1500)
    let sourceData: SCIXyDataSeries = SCIXyDataSeries(XType: .Float, YType: .Float)
    
    var timer: NSTimer!
    var currentIndex: Int32 = 0
    var totalCount = 0.0
    
    var isBeat: Bool = false, lastBeat: Bool = false
    var heartRate: Int = 0
    var lastBeatTime: NSDate = NSDate()
    
    // MARK: Overrided Functions
    
    override func completeConfiguration() {
        super.completeConfiguration()
        addAxis()
        addDefaultModifiers()
        addDataSeries()
    }
    
    override func willMoveToSuperview(newSuperview: UIView?) {
        super.willMoveToSuperview(newSuperview)
        if timer == nil {
            timer = NSTimer.scheduledTimerWithTimeInterval(0.04,
                                                           target: self,
                                                           selector: "updateECGData",
                                                           userInfo: nil,
                                                           repeats: true)
        }
    }
    
    override func removeFromSuperview() {
        super.removeFromSuperview()
        if timer != nil {
            timer.invalidate()
        }
    }
    
    // MARK: Internal Function
    
    func updateECGData() {
        var i = 0
        while i < 10 {
            appendPoint(400)
            i += 1
        }
        
        isBeat = SCIGenericDouble(dataSeries.yValues().valueAt(dataSeries.count()-3)) > 0.5 ||
            SCIGenericDouble(dataSeries.yValues().valueAt(dataSeries.count()-5)) > 0.5 ||
            SCIGenericDouble(dataSeries.yValues().valueAt(dataSeries.count()-8)) > 0.5
        
        if isBeat && !lastBeat {
            heartRate = Int(60.0/lastBeatTime.timeIntervalSinceNow)
            lastBeatTime = NSDate()
        }
        chartSurface.viewportManager .zoomExtents()
        
    }
    
    // MARK: Private Methods
    
    private func addAxis() {
        
        let updateTime = 1.0 / 30.0
        let axisStyle = generateDefaultAxisStyle()
        
        let xAxis = SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisXId)
        xAxis.autoRange = .Always
        xAxis.animatedChangeDuration = updateTime*2
        xAxis.animateVisibleRangeChanges = true
        chartSurface.attachAxis(xAxis, isXAxis: true)
        
        let yAxis = SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisYId)
        yAxis.autoRange = .Always
        yAxis.animatedChangeDuration = updateTime*2
        yAxis.animateVisibleRangeChanges = true
        
        chartSurface.attachAxis(yAxis, isXAxis: false)

    }
    
    private func addDataSeries() {
        DataManager.loadDataFromFile(sourceData, fileName: "WaveformData")
        dataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        let renderableSeries = SCIFastLineRenderableSeries()
        renderableSeries.style.linePen = SCIPenSolid(colorCode: 0xFFf6e8b3, width: 0.5)
        renderableSeries.xAxisId = axisXId
        renderableSeries.yAxisId = axisYId
        renderableSeries.dataSeries = dataSeries
        chartSurface.attachRenderableSeries(renderableSeries)
        chartSurface.invalidateElement()
    }
    
    private func appendPoint(point: Double) {
        if currentIndex >= sourceData.count() {
            currentIndex = 0
        }
        
        let voltage = SCIGenericFloat(sourceData.yValues().valueAt(Int(currentIndex)))*1000
        let time = totalCount / point
        dataSeries.appendX(SCIGeneric(time), y: SCIGeneric(voltage))
        chartSurface.invalidateElement()
        currentIndex += 1
        totalCount += 1
        lastBeat = isBeat
    }
    
}
