//
//  SCSFIFOSpeedTestSciChart.swift
//  SciChartSwiftDemo
//
//  Created by Mykola Hrybeniuk on 6/29/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import UIKit
import SciChart

class SCSFIFOSpeedTestSciChart: SCSBaseChartView, SpeedTest {
    
    let randomWalkGenerator = RandomWalkGenerator()
    
    var dataSeries : SCIXyDataSeries!
    var delegate: DrawingProtocolDelegate!
    var chartProviderName: String! = "SciChart"
    var xCount: Int = 0
    var appendCount: Int32 = 0
    var parameters: TestParameters!
    
    
    // MARK: SpeedTestProtocol
    
    func runTest(testParameters: TestParameters) {
        parameters = testParameters
        appendCount = testParameters.AppendPoints
        
        addSeries()
        
        var i: Int32 = 0;
        while i < parameters.PointCount {
            dataSeries.appendX(SCIGeneric(xCount), y: SCIGeneric(randomWalkGenerator.next(0.0, 1.0, false)))
            xCount += 1
            i += 1
        }
        
        if let renderebleSeries = chartSurface.renderableSeries.firstObject as? SCIFastLineRenderableSeries {
            renderebleSeries.style.linePen = SCIPenSolid(colorCode: 0xFFffffff, width: Float(parameters.StrokeThikness))
        }
        
        chartSurface.invalidateElement()
    }
    
    func updateChart() {
        delegate.chartExampleStarted!()
        
        
        dataSeries.appendX(SCIGeneric(xCount), y: SCIGeneric(randomWalkGenerator.next(0.0, 1.0, false)))
        xCount += 1
        
        chartSurface.invalidateElement()
    }
    
    func stopTest() {
        delegate.processCompleted!()
    }
    
    // MARK: Overrided Functions
    
    override func completeConfiguration() {
        super.completeConfiguration()
        addAxes()
    }
    
    // MARK: Private Functions
    
    private func addAxes() {
        
        let axisStyle = generateDefaultAxisStyle()
        
        let axisX = SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisXId);
        axisX.autoRange = .Always
        axisX.animatedChangeDuration = 1.0/30.0*2
        axisX.animateVisibleRangeChanges = true
        
        chartSurface.attachAxis(axisX, isXAxis: true)
        
        let axisY = SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisYId);
        axisY.autoRange = .Always
        axisY.animatedChangeDuration = 1.0/30.0*2
        axisY.animateVisibleRangeChanges = true
        
        chartSurface.attachAxis(axisY, isXAxis: false)
        
        addDefaultModifiers()
        
    }
    
    private func addSeries() {
        dataSeries = SCIXyDataSeries.init(fifoWithXType: .Float, YType: .Float, fifoSize: parameters.PointCount)
        dataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        let renderSeries = SCIFastLineRenderableSeries()
        renderSeries.dataSeries = dataSeries
        renderSeries.xAxisId = axisXId
        renderSeries.yAxisId = axisYId
        
        chartSurface.attachRenderableSeries(renderSeries)
        
    }
    
}

