//
//  SCSFanChartView.swift
//  SciChartSwiftDemo
//
//  Created by Hrybenuik Mykola on 9/22/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import Foundation
import SciChart

class SCSFanChartView: SCSBaseChartView {

    // MARK: Overrided Functions
    
    override func completeConfiguration() {
        super.completeConfiguration()
        addAxis()
        addDefaultModifiers()
        addDataSeries()
    }
    
    // MARK: Private Methods
    
    private func addAxis() {
        let axisStyle = generateDefaultAxisStyle()
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultDateTimeAxis(withAxisStyle: axisStyle, axisId: axisXId), isXAxis: true)
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisYId), isXAxis: false)
    }
    
    private func addDataSeries() {
        
        let dataSeries = SCIXyDataSeries(XType:.DateTime, YType:.Float)
        dataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()

        let xyyDataSeries = SCIXyyDataSeries(XType:.DateTime, YType:.Float)
        xyyDataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()

        let xyyDataSeries1 = SCIXyyDataSeries(XType:.DateTime, YType:.Float)
        xyyDataSeries1.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        let xyyDataSeries2 = SCIXyyDataSeries(XType:.DateTime, YType:.Float)
        xyyDataSeries2.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        self.generatingDataPoints(10) { (result: SCSDataPoint) in
            dataSeries.appendX(SCIGeneric(result.date), y: SCIGeneric(result.actualValue))
            xyyDataSeries.appendX(SCIGeneric(result.date), y1: SCIGeneric(result.maxValue), y2: SCIGeneric(result.minValue))
            xyyDataSeries1.appendX(SCIGeneric(result.date), y1: SCIGeneric(result.value1), y2: SCIGeneric(result.value4))
            xyyDataSeries2.appendX(SCIGeneric(result.date), y1: SCIGeneric(result.value2), y2: SCIGeneric(result.value3))
        }
      
        let dataRenderSeries = SCIFastLineRenderableSeries()
        dataRenderSeries.dataSeries = dataSeries
        dataRenderSeries.xAxisId = axisXId
        dataRenderSeries.yAxisId = axisYId
        dataRenderSeries.style.linePen = SCIPenSolid(color: UIColor.redColor(), width: 1.0)
        
        chartSurface.attachRenderableSeries(createRenderableSeriesWith(xyyDataSeries))
        chartSurface.attachRenderableSeries(createRenderableSeriesWith(xyyDataSeries1))
        chartSurface.attachRenderableSeries(createRenderableSeriesWith(xyyDataSeries2))
        chartSurface.attachRenderableSeries(dataRenderSeries)
        chartSurface.invalidateElement()
        
    }
    
    private func createRenderableSeriesWith(dataSeries: SCIXyyDataSeries) -> SCIBandRenderableSeries {
        let renderebleDataSeries = SCIBandRenderableSeries()
        renderebleDataSeries.style.brush1 = SCIBrushSolid(color: UIColor.init(red: 1.0, green: 0.4, blue: 0.4, alpha: 0.5))
        renderebleDataSeries.style.brush2 = SCIBrushSolid(color: UIColor.init(red: 1.0, green: 0.4, blue: 0.4, alpha: 0.5))
        renderebleDataSeries.style.pen1 = SCIPenSolid(color: UIColor.greenColor(), width: 0.5)
        renderebleDataSeries.style.pen2 = SCIPenSolid(color: UIColor.clearColor(), width: 0.5)
        renderebleDataSeries.style.drawPointMarkers = false
        renderebleDataSeries.xAxisId = axisXId
        renderebleDataSeries.yAxisId = axisYId
        renderebleDataSeries.dataSeries = dataSeries
        return renderebleDataSeries;
    }
    
    private func generatingDataPoints(count: Int, handler: (SCSDataPoint) -> Void) {
        var dateTime = NSDate()
        var lastValue: Double = 0.0
        var i: Int = 0
        while i <= count {
            let nextValue = lastValue + SCSDataManager.randomize(-0.5, max: 0.5)
            lastValue = nextValue
            dateTime = dateTime.dateByAddingTimeInterval(3600*24)
            
            var dataPoint = SCSDataPoint()
            dataPoint.date = dateTime
            dataPoint.actualValue = nextValue
            if i > 4 {
                dataPoint.maxValue = nextValue + (Double(i) - 5) * 0.3
                dataPoint.value1 = nextValue + (Double(i) - 5) * 0.2
                dataPoint.value2 = nextValue + (Double(i) - 5) * 0.1
                dataPoint.value3 = nextValue - (Double(i) - 5) * 0.1
                dataPoint.value4 = nextValue - (Double(i) - 5) * 0.2
                dataPoint.minValue = nextValue - (Double(i) - 5) * 0.3
            }
            
            handler(dataPoint)
            
            i += 1
        }
        
    }
    
}

public struct SCSDataPoint {
    var maxValue = Double.NaN
    var minValue = Double.NaN
    var value1 = Double.NaN
    var value2 = Double.NaN
    var value3 = Double.NaN
    var value4 = Double.NaN
    var actualValue = Double.NaN
    var date = NSDate()
    
    
}
