//
//  SCSMultiPaneStockChartView.swift
//  SciChartSwiftDemo
//
//  Created by Mykola Hrybeniuk on 7/19/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import Foundation
import SciChart
import Darwin

enum DataSeriesName : Int {
    case CandleData = 1
    case VolumeData
    case RsiData
    case McadColumnData
    case McadBandData
    case HighData
    case LowData
}

class SCSMultiPaneStockChartView: UIView {
    
    let sciChartView1 = SCSBaseChartView()
    let sciChartView2 = SCSBaseChartView()
    let sciChartView3 = SCSBaseChartView()
    let sciChartView4 = SCSBaseChartView()
    let szem = SCIMultiSurfaceModifier(modifierType: SCIZoomExtentsModifier.self)
   
    var dataSource : [SCDMultiPaneItem]
    var dataSeries = [DataSeriesName : SCIDataSeriesProtocol]()
    let rangeSync = SCIAxisRangeSyncronization()
    let axisAreaSync = SCIAxisAreaSizeSyncronization()
    
    override init(frame: CGRect) {
        dataSource = DataManager.loadPaneStockData()
        super.init(frame: frame)
        configureChartSuraface()
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func awakeFromNib() {
        super.awakeFromNib()
        configureChartSuraface()
    }
    
    // MARK: Private Functions
    
    private func configureChartSuraface() {
        
        generateDataSeries()
        
        axisAreaSync.syncMode = .Right
        
        sciChartView1.translatesAutoresizingMaskIntoConstraints = false
        addSubview(sciChartView1)
        
        sciChartView2.translatesAutoresizingMaskIntoConstraints = false
        addSubview(sciChartView2)
        
        sciChartView3.translatesAutoresizingMaskIntoConstraints = false
        addSubview(sciChartView3)
        
        sciChartView4.translatesAutoresizingMaskIntoConstraints = false
        addSubview(sciChartView4)
        
        let layoutDictionary = ["SciChart1" : sciChartView1,
                                "SciChart2" : sciChartView2,
                                "SciChart3" : sciChartView3,
                                "SciChart4" : sciChartView4]
        
        addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("|-(0)-[SciChart1]-(0)-|",
            options: .DirectionLeadingToTrailing,
            metrics: nil,
            views: layoutDictionary))
        
        addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("|-(0)-[SciChart2]-(0)-|",
            options: .DirectionLeadingToTrailing,
            metrics: nil,
            views: layoutDictionary))
        
        addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("|-(0)-[SciChart3]-(0)-|",
            options: .DirectionLeadingToTrailing,
            metrics: nil,
            views: layoutDictionary))
        
        addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("|-(0)-[SciChart4]-(0)-|",
            options: .DirectionLeadingToTrailing,
            metrics: nil,
            views: layoutDictionary))
        
        addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("V:|-(0)-[SciChart1]-(0)-[SciChart2(SciChart3)]-(0)-[SciChart3(SciChart2)]-(0)-[SciChart4(SciChart3)]-(0)-|",
            options: .DirectionLeadingToTrailing,
            metrics: nil,
            views: layoutDictionary))
        
        addConstraint(NSLayoutConstraint(item: sciChartView1,
            attribute: .Height, relatedBy: .Equal, toItem: self,
            attribute: .Height,
            multiplier: 0.5,
            constant: 0))
        

        addAxisToTopChart()
        addAxisForChartView(sciChartView2)
        addAxisForChartView(sciChartView3)
        addAxisForChartView(sciChartView4)
        
        addModifiersForSurface(sciChartView1)
        addModifiersForSurface(sciChartView2)
        addModifiersForSurface(sciChartView3)
        addModifiersForSurface(sciChartView4)
        
        sciChartView1.chartSurface.attachRenderableSeries(generateLineSeries())
        sciChartView1.chartSurface.attachRenderableSeries(generateLineSeries1())
        sciChartView1.chartSurface.attachRenderableSeries(generateCandleStick())
        
        sciChartView2.chartSurface.attachRenderableSeries(generateBandSeries())
        sciChartView2.chartSurface.attachRenderableSeries(generateColumnSeries())
        
        sciChartView3.chartSurface.attachRenderableSeries(generateLineSeries2())
        
        sciChartView4.chartSurface.attachRenderableSeries(generateColumnSeries1())
   
        
        addTextAnnotation(" EUR/USD", forSurface: sciChartView1)
        addTextAnnotation(" MACD", forSurface: sciChartView2)
        addTextAnnotation(" RSI", forSurface: sciChartView3)
        addTextAnnotation(" Volume", forSurface: sciChartView4)
    }
    
    private func addAxisToTopChart() {
        sciChartView1.chartSurface.attachAxis(SCSFactoryAxis.createCategoryDateTimeAxis(withAxisStyle: sciChartView1.generateDefaultAxisStyle(), axisId: sciChartView1.axisXId), isXAxis: true)
        sciChartView1.chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: sciChartView1.generateDefaultAxisStyle(), axisId: sciChartView1.axisYId), isXAxis: false)
        sciChartView1.chartSurface.yAxes.itemAt(0).autoRange = .Always
        sciChartView1.chartSurface.yAxes.itemAt(0).textFormatting = "$%.4f"
        sciChartView1.chartSurface.yAxes.itemAt(0).style.labelStyle.colorCode = 0xFFafb3b6
        sciChartView1.chartSurface.yAxes.itemAt(0).style.labelStyle.fontSize = 12
        sciChartView1.chartSurface.xAxes.itemAt(0).style.labelStyle.colorCode = 0xFFafb3b6
        sciChartView1.chartSurface.xAxes.itemAt(0).style.labelStyle.fontSize = 12
        rangeSync.attachAxis(sciChartView1.chartSurface.xAxes.itemAt(0))
    }
    
    private func addTextAnnotation(text: String, forSurface surfaceView: SCSBaseChartView) {
        
        let textFormatting = SCITextFormattingStyle()
        textFormatting.fontSize = 14
        textFormatting.fontName = "Arial"
        textFormatting.colorCode = 0xFFafb3b6
        
        let textAnnotation = SCITextAnnotation()
        textAnnotation.xAxisId = surfaceView.axisXId
        textAnnotation.yAxisId = surfaceView.axisYId
        textAnnotation.coordMode = .Relative
        textAnnotation.xPosition = SCIGeneric(0.03)
        textAnnotation.yPosition = SCIGeneric(0.2)
        textAnnotation.text = text
        textAnnotation.style.textStyle = textFormatting
        textAnnotation.style.textColor = UIColor.grayColor()
        textAnnotation.style.backgroundColor = UIColor.init(red:22.0 / 255.0, green: 33.0 / 255.0, blue: 40.0 / 255.0, alpha: 1.0)
        
        let annotationGroup = SCIAnnotationGroup(childAnnotations: [textAnnotation])
        surfaceView.chartSurface.annotation = annotationGroup
    }
    
    private func addAxisForChartView(charView: SCSBaseChartView) {
        let xAxis = SCSFactoryAxis.createCategoryDateTimeAxis(withAxisStyle: charView.generateDefaultAxisStyle(), axisId: charView.axisXId);
        xAxis.isVisible = false;
        rangeSync.attachAxis(xAxis)
        let yAxis = SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: charView.generateDefaultAxisStyle(), axisId: charView.axisYId)
        yAxis.autoRange = .Always
        yAxis.style.labelStyle.fontSize = 12
        yAxis.style.labelStyle.colorCode = 0xFFafb3b6
        
        if charView == sciChartView2 {
            yAxis.textFormatting = "%.2f"
        }
        else if charView == sciChartView3 {
            yAxis.textFormatting = "%.1f"
        }
        else if charView == sciChartView4 {
            if let yAxisNumeric = yAxis as? SCINumericAxis {
                yAxisNumeric.numberFormatter = NSNumberFormatter()
                yAxisNumeric.numberFormatter.maximumIntegerDigits = 3
                yAxisNumeric.numberFormatter.numberStyle = .ScientificStyle
                yAxisNumeric.numberFormatter.exponentSymbol = "E+"
            }
        }
        
        charView.chartSurface.attachAxis(xAxis, isXAxis: true)
        charView.chartSurface.attachAxis(yAxis, isXAxis: false)
    }
    
    private func addModifiersForSurface(chartSurfaceView: SCSBaseChartView) {
        let xAxisDragmodifier = SCIXAxisDragModifier()
        xAxisDragmodifier.modifierName = "xAxisDragModifierName"
        xAxisDragmodifier.axisId = chartSurfaceView.axisXId
        xAxisDragmodifier.dragMode = .Scale
        xAxisDragmodifier.clipModeX = .None
        
        let pinchZoomModifier = SCIPinchZoomModifier()
        pinchZoomModifier.modifierName = "pinchZoomModifierName"
        
        let panZoomModifier = SCIZoomPanModifier()
        
        let groupModifier = SCIModifierGroup(childModifiers: [xAxisDragmodifier, pinchZoomModifier, szem, panZoomModifier])
        
        chartSurfaceView.chartSurface.chartModifier = groupModifier
        
        axisAreaSync.attachSurface(chartSurfaceView.chartSurface)
    }
    
    private func generateLineSeries() -> SCIRenderableSeriesBase {
        let renderebleSeries = SCIFastLineRenderableSeries()
        renderebleSeries.style.linePen = SCIPenSolid.init(color: UIColor.greenColor(), width: 0.5)
        renderebleSeries.dataSeries = dataSeries[.HighData]
        renderebleSeries.xAxisId = sciChartView1.axisXId
        renderebleSeries.yAxisId = sciChartView1.axisYId
        return renderebleSeries;
    }
    
    private func generateLineSeries1() -> SCIRenderableSeriesBase {
        let renderebleSeries = SCIFastLineRenderableSeries()
        renderebleSeries.style.linePen = SCIPenSolid.init(color: UIColor.redColor(), width: 0.5)
        renderebleSeries.dataSeries = dataSeries[.LowData]
        renderebleSeries.xAxisId = sciChartView1.axisXId
        renderebleSeries.yAxisId = sciChartView1.axisYId
        return renderebleSeries;
    }
    
    private func generateCandleStick() -> SCIRenderableSeriesBase {
        let renderebleSeries = SCIFastCandlestickRenderableSeries()
        renderebleSeries.dataSeries = dataSeries[.CandleData]
        renderebleSeries.xAxisId = sciChartView1.axisXId
        renderebleSeries.yAxisId = sciChartView1.axisYId
        renderebleSeries.style.upWickPen = SCIPenSolid.init(color: UIColor.whiteColor(), width: 0.5)
        renderebleSeries.style.upBodyBrush = SCIBrushSolid.init(color: UIColor.whiteColor())
        let downColor = UIColor(red: 56.0/255.0, green: 110.0/255.0, blue: 165.0/255.0, alpha: 1.0)
        renderebleSeries.style.downBodyBrush = SCIBrushSolid.init(color: downColor)
        renderebleSeries.style.downWickPen = SCIPenSolid.init(color: downColor, width: 0.5)
        return renderebleSeries;
    }
    
    private func generateLineSeries2() -> SCIRenderableSeriesBase {
        let renderebleSeries = SCIFastLineRenderableSeries()
        renderebleSeries.style.linePen = SCIPenSolid.init(color: UIColor(red: 168.0/255.0, green: 202.0/255.0, blue: 230.0/255.0, alpha: 1.0), width: 0.5)
        renderebleSeries.dataSeries = dataSeries[.RsiData]
        renderebleSeries.xAxisId = sciChartView1.axisXId
        renderebleSeries.yAxisId = sciChartView1.axisYId
        return renderebleSeries;
    }
    
    private func generateColumnSeries() -> SCIRenderableSeriesBase {
        let renderebleSeries = SCIFastColumnRenderableSeries()
        renderebleSeries.style.borderPen = SCIPenSolid.init(color: UIColor.whiteColor(), width: 0.5)
        renderebleSeries.style.fillBrush = SCIBrushSolid.init(color: UIColor.whiteColor())
        renderebleSeries.style.dataPointWidth = 0.3
        renderebleSeries.dataSeries = dataSeries[.McadColumnData]
        renderebleSeries.xAxisId = sciChartView2.axisXId
        renderebleSeries.yAxisId = sciChartView2.axisYId
        return renderebleSeries;
    }
    
    private func generateBandSeries() -> SCIRenderableSeriesBase {
        let renderebleSeries = SCIBandRenderableSeries()
        var color = UIColor(red: 69.0/255.0, green: 199.0/255.0, blue: 66.0/255.0, alpha: 1.0)
        renderebleSeries.style.pen1 = SCIPenSolid.init(color: color, width: 0.5)
        renderebleSeries.style.pen2 = SCIPenSolid.init(color: color, width: 0.5)
        color = UIColor(red: 217.0/255.0, green: 77.0/255.0, blue: 82.0/255.0, alpha: 1.0)
        renderebleSeries.style.brush1 = SCIBrushSolid.init(color: color)
        renderebleSeries.style.brush2 = SCIBrushSolid.init(color: color)
        renderebleSeries.style.drawPointMarkers = false
        renderebleSeries.dataSeries = dataSeries[.McadBandData]
        renderebleSeries.xAxisId = sciChartView2.axisXId
        renderebleSeries.yAxisId = sciChartView2.axisYId
        return renderebleSeries;
    }
    
    private func generateColumnSeries1() -> SCIRenderableSeriesBase {
        let renderebleSeries = SCIFastColumnRenderableSeries()
        renderebleSeries.style.borderPen = SCIPenSolid.init(color: UIColor.whiteColor(), width: 0.5)
        renderebleSeries.style.fillBrush = SCIBrushSolid.init(color: UIColor.whiteColor())
        renderebleSeries.style.dataPointWidth = 0.3
        renderebleSeries.dataSeries = dataSeries[.VolumeData]
        renderebleSeries.xAxisId = sciChartView4.axisXId
        renderebleSeries.yAxisId = sciChartView4.axisYId
        return renderebleSeries;
    }
    
    private func generateDataSeries() {
        
        let priceDataSeries = SCIOhlcDataSeries(categoryXType: .DateTime, YType: .Double)
        priceDataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        let columnDataSeries = SCIXyDataSeries(categoryXType: .DateTime, YType: .Double)
        columnDataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        let rsiDataSeries = SCIXyDataSeries(categoryXType: .DateTime, YType: .Double)
        rsiDataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        let columnMcadDataSeries = SCIXyDataSeries(categoryXType: .DateTime, YType: .Double)
        columnMcadDataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        let bandMcadDataSeries = SCIXyyDataSeries(categoryXType: .DateTime, YType: .Double)
        bandMcadDataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        let highDataSeries = SCIXyDataSeries(categoryXType: .DateTime, YType: .Double)
        highDataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        let lowDataSeries = SCIXyDataSeries(categoryXType: .DateTime, YType: .Double)
        lowDataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        let averageGainRsi = SCDMovingAverage(length: 14)
        let averageLossRsi = SCDMovingAverage(length: 14)
        let averageSLow = SCDMovingAverage(length: 12)
        let averageFast = SCDMovingAverage(length: 26)
        let averageSignal = SCDMovingAverage(length: 9)
        let averageLow = SCDMovingAverage(length: 50)
        let averageHigh = SCDMovingAverage(length: 200)
        var previous = self.dataSource.first
        
        for item: SCDMultiPaneItem in self.dataSource {
            let date = SCIGeneric(item.dateTime)
            let open = SCIGeneric(item.open)
            let high = SCIGeneric(item.high)
            let low = SCIGeneric(item.low)
            let close = SCIGeneric(item.close)
            let volume = SCIGeneric(item.volume)
            priceDataSeries.appendX(date,
                                    open: open,
                                    high: high,
                                    low: low,
                                    close: close)
            columnDataSeries.appendX(date, y: volume)
            let rsiValue = self.rsiForAverageGain(averageGainRsi, andAveLoss: averageLossRsi, previous: previous!, currentItem: item)
            let rsi = SCIGeneric(rsiValue)
            rsiDataSeries.appendX(date, y: rsi)
            let mcadPoint = self.mcadPointForSlow(averageSLow, forFast: averageFast, forSignal: averageSignal, andCloseValue: item.close)

            if mcadPoint.divergence.isNaN {
                mcadPoint.divergence = 0.00000000000000;
            }
            columnMcadDataSeries.appendX(date, y: SCIGeneric(mcadPoint.divergence))
            
            bandMcadDataSeries.appendX(date, y1: SCIGeneric(mcadPoint.mcad), y2: SCIGeneric(mcadPoint.signal))
            highDataSeries.appendX(date, y: SCIGeneric(averageHigh.push(item.close).current))
            lowDataSeries.appendX(date, y: SCIGeneric(averageLow.push(item.close).current))
            previous = item
        }
        dataSeries[.CandleData] = priceDataSeries
        dataSeries[.VolumeData] = columnDataSeries
        dataSeries[.RsiData] = rsiDataSeries
        dataSeries[.McadColumnData] = columnMcadDataSeries
        dataSeries[.McadBandData] = bandMcadDataSeries
        dataSeries[.HighData] = highDataSeries
        dataSeries[.LowData] = lowDataSeries
    }
    
    private func rsiForAverageGain(averageGain: SCDMovingAverage, andAveLoss averageLoss: SCDMovingAverage, previous: SCDMultiPaneItem, currentItem item: SCDMultiPaneItem) -> Double {
        let gain = item.close > previous.close ? item.close - previous.close : 0.0
        let loss = previous.close > item.close ? previous.close - item.close : 0.0
        averageGain.push(gain)
        averageLoss.push(loss)
        let relativeStrength = averageGain.current.isNaN || averageLoss.current.isNaN ? Double.NaN : averageGain.current / averageLoss.current
        return relativeStrength.isNaN ? Double.NaN : 100.0 - (100.0 / (1.0 + relativeStrength))
    }
    
    private func mcadPointForSlow(slow: SCDMovingAverage, forFast fast: SCDMovingAverage, forSignal signal: SCDMovingAverage, andCloseValue close: Double) -> SCDMcadPointItem {
        slow.push(close)
        fast.push(close)
        let macd = slow.current - fast.current
        let signalLine = macd.isNaN ? Double.NaN : signal.push(macd).current
        let divergence = macd.isNaN || signalLine.isNaN ? Double.NaN : macd - signalLine
        let point = SCDMcadPointItem()
        point.mcad = macd
        point.signal = signalLine
        point.divergence = divergence
        return point
    }
    

}
