//
//  SCSPalettedChartView.swift
//  SciChartSwiftDemo
//
//  Created by Admin on 08/22/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import UIKit
import SciChart

class MinMaxPaletteProvider: SCIPaletteProvider {
    
    var _styleMin : SCILineSeriesStyle
    var _styleMax : SCILineSeriesStyle
    var _minIndex : Int32
    var _maxIndex : Int32
    
    override init() {
        _styleMin = SCILineSeriesStyle()
        _styleMin.linePen = SCIPenSolid(colorCode: 0xFF99EE99, width: 0.7)
        _styleMin.drawPointMarkers = true
        let minMarker = SCIEllipsePointMarker()
        minMarker.fillBrush = SCIBrushSolid(color: UIColor.cyanColor())
        _styleMin.pointMarker = minMarker
        
        _styleMax = SCILineSeriesStyle()
        _styleMax.linePen = SCIPenSolid(colorCode: 0xFF99EE99, width: 0.7)
        _styleMax.drawPointMarkers = true
        let maxMarker = SCIEllipsePointMarker()
        maxMarker.fillBrush = SCIBrushSolid(color: UIColor.redColor())
        _styleMax.pointMarker = maxMarker
        
        _minIndex = 0
        _maxIndex = 0
    }
    
    override func updateData(data: SCIRenderPassDataProtocol!) {
        var minIndex : Int32 = 0;
        var minValue : Double = DBL_MAX;
        var maxIndex : Int32 = 0;
        var maxValue : Double = -DBL_MAX;
        let points : SCIPointSeriesProtocol = data.pointSeries();
        let count = points.count();
        for i in 0..<count {
            let value: Double = SCIGenericDouble( points.yValues().valueAt( Int(i) ) );
            if (value < minValue) {
                minValue = value;
                minIndex = SCIGenericInt( points.indices().valueAt( Int(i) ) );
            }
            if (value > maxValue) {
                maxValue = value;
                maxIndex = SCIGenericInt( points.indices().valueAt( Int(i) ) );
            }
        }
        _minIndex = minIndex;
        _maxIndex = maxIndex;
    }
    
    override func styleForX(x: Double, y: Double, index: Int32) -> SCIStyle! {
        if (index == _minIndex) {
            return _styleMin;
        } else if (index == _maxIndex) {
            return _styleMax;
        } else {
            return nil;
        }
    }
}

class ZeroLinePaletteProvider: SCIPaletteProvider {
    var _style : SCILineSeriesStyle
    var _zeroLine : Double
    var _yCoordCalc : SCICoordinateCalculator!
    
    override init() {
        _style = SCILineSeriesStyle()
        _style.drawPointMarkers = false
        _style.linePen = SCIPenSolid(color: UIColor.blueColor(), width: 0.7)
        
        _zeroLine = 10
        
        _yCoordCalc = nil
    }
    
    override func updateData(data: SCIRenderPassDataProtocol!) {
         _yCoordCalc = data.yCoordinateCalculator()
    }
    
    override func styleForX(x: Double, y: Double, index: Int32) -> SCIStyle! {
        let value : Double = _yCoordCalc.getDataValueFrom(y)
        if (value < _zeroLine) {
            return _style
        } else  {
            return nil
        }
    }
}

class SCSPalettedChartView: SCSBaseChartView {
    
    // MARK: Overrided Functions
    
    override func completeConfiguration() {
        super.completeConfiguration()
        addAxes()
        addSeries()
    }
    
    // MARK: Private Functions

    private func addAxes() {
 
        let axisStyle = generateDefaultAxisStyle()
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisXId), isXAxis: true)
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisYId), isXAxis: false)
        addDefaultModifiers()

    }
    
    private func addSeries() {
        
        
        let dataSeries = SCIXyDataSeries(XType: .Float, YType: .Float)
        SCSDataManager.putDataInto(dataSeries)
        
        
        let fourierDataSeries = SCIXyDataSeries(XType: .Float, YType: .Float)
        SCSDataManager.putFourierDataInto(fourierDataSeries)
        
        dataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        fourierDataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        let ellipsePointMarker = SCIEllipsePointMarker()
        ellipsePointMarker.drawBorder = true
        ellipsePointMarker.fillBrush = SCIBrushSolid(colorCode: 0xFFd6ffd7)
        ellipsePointMarker.height = 5
        ellipsePointMarker.width = 5
        
        let renderSeries = SCIFastLineRenderableSeries()
        renderSeries.dataSeries = dataSeries
        renderSeries.xAxisId = axisXId
        renderSeries.yAxisId = axisYId
        renderSeries.style.pointMarker = ellipsePointMarker
        renderSeries.style.drawPointMarkers = true
        renderSeries.style.linePen = SCIPenSolid(colorCode: 0xFF99EE99, width: 0.7)
        chartSurface.attachRenderableSeries(renderSeries)
        
        let fourierRenderSeries = SCIFastLineRenderableSeries()
        fourierRenderSeries.dataSeries = fourierDataSeries
        fourierRenderSeries.xAxisId = axisXId
        fourierRenderSeries.yAxisId = axisYId
        fourierRenderSeries.style.linePen = SCIPenSolid(colorCode: 0xFF4c8aff, width: 0.7)
        chartSurface.attachRenderableSeries(fourierRenderSeries)
        
        renderSeries.paletteProvider = MinMaxPaletteProvider()
        fourierRenderSeries.paletteProvider = ZeroLinePaletteProvider()
        
        chartSurface.invalidateElement()
    }
    
}
