//
//  SCSUsingRolloverModifierChartView.swift
//  SciChartSwiftDemo
//
//  Created by Yaroslav Pelyukh on 7/23/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import Foundation
import SciChart

class SCSUsingRolloverModifierChartView: UIView {
    
    let sciChartView = SCSBaseChartView()
    var controlPanel : SCSInterpolationTurnOnOff!
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        completeConfiguration()
    }
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
    }
    
    // MARK: Overrided Functions
    
    override func awakeFromNib() {
        super.awakeFromNib()
        completeConfiguration()
    }
    
    // MARK: Private Functions
    
    func completeConfiguration() {
        configureChartSuraface()
        addAxes()
        addSeries()
        sciChartView.addDefaultModifiers()
    }
    
    private func addPanel() {
        let panel = NSBundle.mainBundle().loadNibNamed("SCSInterpolationTurnOnOff",
                                                       owner: self,
                                                       options: nil)!.first
        
        if let panelValid = panel as? SCSInterpolationTurnOnOff {
            
            controlPanel = panelValid
            
            controlPanel.translatesAutoresizingMaskIntoConstraints = false
            
            controlPanel.useInterpolationClicked.addTarget(self,
                                               action: "turnOnOffInterpolation",
                                               forControlEvents: .TouchUpInside)
            
            addSubview(controlPanel)
            
        }
    }
    
    
    func turnOnOffInterpolation(){
        for series in sciChartView.chartSurface.renderableSeries {
            series.hitTestProvider().hitTestMode = series.hitTestProvider().hitTestMode == .Vertical ? .VerticalInterpolate : .Vertical
        }
    }

    
    private func configureChartSuraface() {
        
        sciChartView.translatesAutoresizingMaskIntoConstraints = false
        self.addSubview(sciChartView)
        
        addPanel()
        
        let layoutDictionary = ["SciChart" : sciChartView, "Panel" : controlPanel]
        
        self.addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("V:|-(0)-[Panel(43)]-(0)-[SciChart]-(0)-|",
            options: .DirectionLeadingToTrailing,
            metrics: nil,
            views: layoutDictionary))
        
        self.addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("|-(0)-[SciChart]-(0)-|",
            options: .DirectionLeadingToTrailing,
            metrics: nil,
            views: layoutDictionary))
        
        self.addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("|-(0)-[Panel]-(0)-|",
            options: .DirectionLeadingToTrailing,
            metrics: nil,
            views: layoutDictionary))
    }
    
    // MARK: Private Functions
    
    private func addAxes() {
        
        let axisStyle = sciChartView.generateDefaultAxisStyle()
        sciChartView.chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: sciChartView.axisXId), isXAxis: true)
        sciChartView.chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: sciChartView.axisYId), isXAxis: false)
        sciChartView.addDefaultModifiers()
        
    }
    
    private func addSeries() {
        let ellipsePointMarker: SCIEllipsePointMarker = SCIEllipsePointMarker()
        ellipsePointMarker.drawBorder = true
        ellipsePointMarker.fillBrush = SCIBrushSolid(colorCode: 0xFFd6ffd7)
        ellipsePointMarker.height = 5
        ellipsePointMarker.width = 5

        
        sciChartView.chartSurface.attachRenderableSeries(self.getFastLineRenderableSeries(ellipsePointMarker, amplitude:1.0 , colorCode:0xFFd7b9a1))
        sciChartView.chartSurface.attachRenderableSeries(self.getFastLineRenderableSeries(ellipsePointMarker, amplitude:0.5 , colorCode:0xFF00540b))
        sciChartView.chartSurface.attachRenderableSeries(self.getFastLineRenderableSeries(nil, amplitude: 0, colorCode: 0xFFa66e38))

        sciChartView.chartSurface.invalidateElement()
        
    }
    
    private func getFastLineRenderableSeries(pointMarker: SCIEllipsePointMarker?, amplitude: Double, colorCode: uint) -> SCIFastLineRenderableSeries {
        let fourierDataSeries: SCIXyDataSeries = SCIXyDataSeries(XType: .Float, YType: .Float)
        let count: Double = 100.0
        let k: Double = 2 * M_PI / 30.0
        
        for i in 0..<Int(count) {
            let phi: Double = k * Double(i)
            fourierDataSeries.appendX(SCIGeneric(i), y: SCIGeneric((amplitude + Double(i) / count) * sin(phi)))
        }
        fourierDataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        let fourierRenderableSeries: SCIFastLineRenderableSeries = SCIFastLineRenderableSeries()
        fourierRenderableSeries.style.linePen = SCIPenSolid(colorCode: colorCode, width: 1.0)
        fourierRenderableSeries.xAxisId = "xAxis"
        fourierRenderableSeries.yAxisId = "yAxis"
        fourierRenderableSeries.dataSeries = fourierDataSeries
        
        fourierRenderableSeries.hitTestProvider().hitTestMode = .VerticalInterpolate;
        
        if pointMarker != nil {
            fourierRenderableSeries.style.drawPointMarkers = true
            fourierRenderableSeries.style.pointMarker = pointMarker
        }
        return fourierRenderableSeries
    }
    
}
